@@ -1,36 +1,45 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, division, print_function
-
 import copy
-import sys
 import threading
+import typing
 import warnings
-from typing import Any, Optional, Tuple
+from os import cpu_count
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    NewType,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 import numpy as np
 
-from .. import _core
+import boost_histogram
+import boost_histogram._core as _core
+
 from .axestuple import AxesTuple
 from .axis import Axis
 from .enum import Kind
-from .kwargs import KWArgs
-from .sig_tools import inject_signature
-from .six import string_types
 from .storage import Double, Storage
-from .utils import MAIN_FAMILY, cast, register, set_family, set_module
-from .view import _to_view
-
-if sys.version_info >= (3, 4):
-    from os import cpu_count
-else:
-    from multiprocessing import cpu_count
-
-ArrayLike = Any
+from .typing import Accumulator, ArrayLike, CppHistogram, SupportsIndex
+from .utils import cast, register, set_module
+from .view import MeanView, WeightedMeanView, WeightedSumView, _to_view
 
+if TYPE_CHECKING:
+    from builtins import ellipsis
 
 NOTHING = object()
 
-_histograms = (
+
+_histograms: Set[Type[CppHistogram]] = {
     _core.hist.any_double,
     _core.hist.any_int64,
     _core.hist.any_atomic_int64,
@@ -38,36 +47,45 @@ _histograms = (
     _core.hist.any_weight,
     _core.hist.any_mean,
     _core.hist.any_weighted_mean,
-)
+}
 
 
-def _fill_cast(value, inner=False):
+CppAxis = NewType("CppAxis", object)
+
+InnerIndexing = Union[SupportsIndex, Callable[[Axis], int], slice, "ellipsis"]
+IndexingWithMapping = Union[InnerIndexing, Mapping[int, InnerIndexing]]
+IndexingExpr = Union[IndexingWithMapping, Tuple[IndexingWithMapping, ...]]
+
+T = TypeVar("T")
+
+
+def _fill_cast(value: T, *, inner: bool = False) -> Union[T, np.ndarray, Tuple[T, ...]]:
     """
     Convert to NumPy arrays. Some buffer objects do not get converted by forcecast.
     If not called by itself (inner=False), then will work through one level of tuple/list.
     """
-    if value is None or isinstance(value, string_types + (bytes,)):
-        return value
+    if value is None or isinstance(value, (str, bytes)):
+        return value  # type: ignore
     elif not inner and isinstance(value, (tuple, list)):
-        return tuple(_fill_cast(a, inner=True) for a in value)
+        return tuple(_fill_cast(a, inner=True) for a in value)  # type: ignore
     elif hasattr(value, "__iter__") or hasattr(value, "__array__"):
         return np.asarray(value)
     else:
         return value
 
 
-def _arg_shortcut(item):
+def _arg_shortcut(item: Union[Tuple[int, float, float], Axis, CppAxis]) -> CppAxis:
     msg = "Developer shortcut: will be removed in a future version"
     if isinstance(item, tuple) and len(item) == 3:
         warnings.warn(msg, FutureWarning)
-        return _core.axis.regular_uoflow(item[0], item[1], item[2])
+        return _core.axis.regular_uoflow(item[0], item[1], item[2])  # type: ignore
     elif isinstance(item, Axis):
-        return item._ax
+        return item._ax  # type: ignore
     else:
         raise TypeError("Only axes supported in histogram constructor")
 
 
-def _expand_ellipsis(indexes, rank):
+def _expand_ellipsis(indexes: Iterable[Any], rank: int) -> List[Any]:
     indexes = list(indexes)
     number_ellipses = indexes.count(Ellipsis)
     if number_ellipses == 0:
@@ -85,12 +103,14 @@ def _expand_ellipsis(indexes, rank):
         raise IndexError("an index can only have a single ellipsis ('...')")
 
 
+H = TypeVar("H", bound="Histogram")
+
+
 # We currently do not cast *to* a histogram, but this is consistent
 # and could be used later.
-@register(_histograms)
-@set_family(MAIN_FAMILY)
+@register(_histograms)  # type: ignore
 @set_module("boost_histogram")
-class Histogram(object):
+class Histogram:
     # Note this is a __slots__ __dict__ class!
     __slots__ = (
         "_hist",
@@ -99,10 +119,42 @@ class Histogram(object):
     )
     # .metadata and ._variance_known are part of the dict
 
-    @inject_signature(
-        "self, *axes, storage=Double(), metadata=None", locals={"Double": Double}
-    )
-    def __init__(self, *axes, **kwargs):
+    _family: object = boost_histogram
+
+    def __init_subclass__(cls, *, family: Optional[object] = None) -> None:
+        """
+        Sets the family for the histogram. This should be a unique object (such
+        as the main module of your package) that is consistently set across all
+        subclasses. When converting back from C++, casting will try to always
+        pick the best matching family from the loaded subclasses for Axis and
+        such.
+        """
+        super().__init_subclass__()
+        cls._family = family if family is not None else object()
+
+    @typing.overload
+    def __init__(self, *args: "Histogram") -> None:
+        ...
+
+    @typing.overload
+    def __init__(self, *args: CppHistogram, metadata: Any = ...) -> None:
+        ...
+
+    @typing.overload
+    def __init__(
+        self,
+        *axes: Union[Axis, CppAxis],
+        storage: Storage,
+        metadata: Any = None,
+    ) -> None:
+        ...
+
+    def __init__(
+        self,
+        *axes: Union[Axis, CppAxis, "Histogram", CppHistogram],
+        storage: Storage = Double(),  # noqa: B008
+        metadata: Any = None,
+    ) -> None:
         """
         Construct a new histogram.
 
@@ -122,9 +174,9 @@ class Histogram(object):
         self._variance_known = True
 
         # Allow construction from a raw histogram object (internal)
-        if len(axes) == 1 and isinstance(axes[0], _histograms):
-            self._hist = axes[0]
-            self.metadata = kwargs.get("metadata")
+        if len(axes) == 1 and isinstance(axes[0], tuple(_histograms)):
+            self._hist: Any = axes[0]
+            self.metadata = metadata
             self.axes = self._generate_axes_()
             return
 
@@ -138,15 +190,13 @@ class Histogram(object):
 
         # Support objects that provide a to_boost method, like Uproot
         elif len(axes) == 1 and hasattr(axes[0], "_to_boost_histogram_"):
-            self.__init__(axes[0]. _to_boost_histogram_())
+            self.__init__(axes[0]._to_boost_histogram_())  # type: ignore
             return
 
-        # Keyword only trick (change when Python2 is dropped)
-        with KWArgs(kwargs) as k:
-            storage = k.optional("storage")
-            if storage is None:
-                storage = Double()
-            self.metadata = k.optional("metadata")
+        if storage is None:
+            storage = Double()
+
+        self.metadata = metadata
 
         # Check for missed parenthesis or incorrect types
         if not isinstance(storage, Storage):
@@ -158,11 +208,11 @@ class Histogram(object):
                 raise KeyError("Only storages allowed in storage argument")
 
         # Allow a tuple to represent a regular axis
-        axes = tuple(_arg_shortcut(arg) for arg in axes)
+        axes = tuple(_arg_shortcut(arg) for arg in axes)  # type: ignore
 
         if len(axes) > _core.hist._axes_limit:
             raise IndexError(
-                "Too many axes, must be less than {}".format(_core.hist._axes_limit)
+                f"Too many axes, must be less than {_core.hist._axes_limit}"
             )
 
         # Check all available histograms, and if the storage matches, return that one
@@ -174,9 +224,10 @@ class Histogram(object):
 
         raise TypeError("Unsupported storage")
 
-    def _from_histogram_object(self, other):
+    def _from_histogram_object(self, other: "Histogram") -> None:
         """
-        Return a new histogram object, possibly converting from a different subclass.
+        Convert self into a new histogram object based on another, possibly
+        converting from a different subclass.
         """
         self._hist = other._hist
         self.__dict__ = copy.copy(other.__dict__)
@@ -188,7 +239,7 @@ class Histogram(object):
         other._export_bh_(self)
         self._import_bh_()
 
-    def _import_bh_(self):
+    def _import_bh_(self) -> None:
         """
         If any post-processing is needed to pass a histogram between libraries, a
         subclass can implement it here. self is the new instance in the current
@@ -196,14 +247,14 @@ class Histogram(object):
         """
 
     @classmethod
-    def _export_bh_(cls, self):
+    def _export_bh_(cls, self: "Histogram") -> None:
         """
         If any preparation is needed to pass a histogram between libraries, a subclass can
         implement it here. cls is the current class being converted from, and self is the
         instance in the class being converted to.
         """
 
-    def _generate_axes_(self):
+    def _generate_axes_(self) -> AxesTuple:
         """
         This is called to fill in the axes. Subclasses can override it if they need
         to change the axes tuple.
@@ -211,7 +262,7 @@ class Histogram(object):
 
         return AxesTuple(self._axis(i) for i in range(self.ndim))
 
-    def _new_hist(self, _hist, memo=NOTHING):
+    def _new_hist(self: H, _hist: CppHistogram, memo: Any = NOTHING) -> H:
         """
         Return a new histogram given a new _hist, copying metadata.
         """
@@ -232,26 +283,34 @@ class Histogram(object):
         return other
 
     @property
-    def ndim(self):
+    def ndim(self) -> int:
         """
         Number of axes (dimensions) of the histogram.
         """
-        return self._hist.rank()
+        return self._hist.rank()  # type: ignore
 
-    def view(self, flow=False):
+    def view(
+        self, flow: bool = False
+    ) -> Union[np.ndarray, WeightedSumView, WeightedMeanView, MeanView]:
         """
         Return a view into the data, optionally with overflow turned on.
         """
         return _to_view(self._hist.view(flow))
 
-    def __array__(self):
+    def __array__(self) -> np.ndarray:
         return self.view(False)
 
-    def __add__(self, other):
+    def __eq__(self, other: Any) -> bool:
+        return hasattr(other, "_hist") and self._hist == other._hist
+
+    def __ne__(self, other: Any) -> bool:
+        return (not hasattr(other, "_hist")) or self._hist != other._hist
+
+    def __add__(self: H, other: Union["Histogram", np.ndarray, float]) -> H:
         result = self.copy(deep=False)
         return result.__iadd__(other)
 
-    def __iadd__(self, other):
+    def __iadd__(self: H, other: Union["Histogram", np.ndarray, float]) -> H:
         if isinstance(other, (int, float)) and other == 0:
             return self
         self._compute_inplace_op("__iadd__", other)
@@ -261,37 +320,44 @@ class Histogram(object):
 
         return self
 
-    def __radd__(self, other):
+    def __radd__(self: H, other: Union["Histogram", np.ndarray, float]) -> H:
         return self + other
 
-    def __eq__(self, other):
-        return self._hist == other._hist
-
-    def __ne__(self, other):
-        return self._hist != other._hist
-
     # If these fail, the underlying object throws the correct error
-    def __mul__(self, other):
+    def __mul__(self: H, other: Union["Histogram", np.ndarray, float]) -> H:
         result = self.copy(deep=False)
         return result._compute_inplace_op("__imul__", other)
 
-    def __rmul__(self, other):
+    def __rmul__(self: H, other: Union["Histogram", np.ndarray, float]) -> H:
         return self * other
 
-    def __truediv__(self, other):
+    def __truediv__(self: H, other: Union["Histogram", np.ndarray, float]) -> H:
         result = self.copy(deep=False)
         return result._compute_inplace_op("__itruediv__", other)
 
-    def __div__(self, other):
+    def __div__(self: H, other: Union["Histogram", np.ndarray, float]) -> H:
         result = self.copy(deep=False)
         return result._compute_inplace_op("__idiv__", other)
 
-    def _compute_inplace_op(self, name, other):
+    def __idiv__(self: H, other: Union["Histogram", np.ndarray, float]) -> H:
+        return self._compute_inplace_op("__idiv__", other)
+
+    def __itruediv__(self: H, other: Union["Histogram", np.ndarray, float]) -> H:
+        return self._compute_inplace_op("__itruediv__", other)
+
+    def __imul__(self: H, other: Union["Histogram", np.ndarray, float]) -> H:
+        return self._compute_inplace_op("__imul__", other)
+
+    def _compute_inplace_op(
+        self: H, name: str, other: Union["Histogram", np.ndarray, float]
+    ) -> H:
+        # Also takes CppHistogram, but that confuses mypy because it's hard to pick out
         if isinstance(other, Histogram):
             getattr(self._hist, name)(other._hist)
-        elif isinstance(other, _histograms):
+        elif isinstance(other, tuple(_histograms)):
             getattr(self._hist, name)(other)
-        elif hasattr(other, "shape") and other.shape:
+        elif hasattr(other, "shape") and other.shape:  # type: ignore
+            assert not isinstance(other, float)
             if len(other.shape) != self.ndim:
                 raise ValueError(
                     "Number of dimensions {} must match histogram {}".format(
@@ -316,18 +382,14 @@ class Histogram(object):
         self._variance_known = False
         return self
 
-    def __idiv__(self, other):
-        return self._compute_inplace_op("__idiv__", other)
-
-    def __itruediv__(self, other):
-        return self._compute_inplace_op("__itruediv__", other)
-
-    def __imul__(self, other):
-        return self._compute_inplace_op("__imul__", other)
-
     # TODO: Marked as too complex by flake8. Should be factored out a bit.
-    @inject_signature("self, *args, weight=None, sample=None, threads=None")
-    def fill(self, *args, **kwargs):  # noqa: C901
+    def fill(
+        self: H,
+        *args: Union[ArrayLike, str],
+        weight: Optional[ArrayLike] = None,
+        sample: Optional[ArrayLike] = None,
+        threads: Optional[int] = None,
+    ) -> H:  # noqa: C901
         """
         Insert data into the histogram.
 
@@ -335,7 +397,7 @@ class Histogram(object):
         ----------
         *args : Union[Array[float], Array[int], Array[str], float, int, str]
             Provide one value or array per dimension.
-        weight : List[Union[Array[float], Array[int], Array[str], float, int, str]]]
+        weight : List[Union[Array[float], Array[int], float, int, str]]]
             Provide weights (only if the histogram storage supports it)
         sample : List[Union[Array[float], Array[int], Array[str], float, int, str]]]
             Provide samples (only if the histogram storage supports it)
@@ -345,11 +407,6 @@ class Histogram(object):
             available threads (usually two per core).
         """
 
-        with KWArgs(kwargs) as kw:
-            weight = kw.optional("weight")
-            sample = kw.optional("sample")
-            threads = kw.optional("threads")
-
         if (
             self._hist._storage_type
             not in {
@@ -362,12 +419,12 @@ class Histogram(object):
             self._variance_known = False
 
         # Convert to NumPy arrays
-        args = _fill_cast(args)
-        weight = _fill_cast(weight)
-        sample = _fill_cast(sample)
+        args_ars = _fill_cast(args)
+        weight_ars = _fill_cast(weight)
+        sample_ars = _fill_cast(sample)
 
         if threads is None or threads == 1:
-            self._hist.fill(*args, weight=weight, sample=sample)
+            self._hist.fill(*args_ars, weight=weight_ars, sample=sample_ars)
             return self
 
         if threads == 0:
@@ -379,27 +436,37 @@ class Histogram(object):
         }:
             raise RuntimeError("Mean histograms do not support threaded filling")
 
-        data = [np.array_split(a, threads) for a in args]
+        data = [np.array_split(a, threads) for a in args_ars]
 
         if weight is None or np.isscalar(weight):
-            weights = [weight] * threads
+            assert threads is not None
+            weights = [weight_ars] * threads
         else:
-            weights = np.array_split(weight, threads)
+            weights = np.array_split(weight_ars, threads)
 
-        if sample is None or np.isscalar(sample):
-            samples = [sample] * threads
+        if sample_ars is None or np.isscalar(sample_ars):
+            assert threads is not None
+            samples = [sample_ars] * threads
         else:
-            samples = np.array_split(sample, threads)
+            samples = np.array_split(sample_ars, threads)
 
         if self._hist._storage_type is _core.storage.atomic_int64:
 
-            def fun(weight, sample, *args):
+            def fun(
+                weight: Optional[ArrayLike],
+                sample: Optional[ArrayLike],
+                *args: np.ndarray,
+            ) -> None:
                 self._hist.fill(*args, weight=weight, sample=sample)
 
         else:
             sum_lock = threading.Lock()
 
-            def fun(weight, sample, *args):
+            def fun(
+                weight: Optional[ArrayLike],
+                sample: Optional[ArrayLike],
+                *args: np.ndarray,
+            ) -> None:
                 local_hist = self._hist.__copy__()
                 local_hist.reset()
                 local_hist.fill(*args, weight=weight, sample=sample)
@@ -419,7 +486,7 @@ class Histogram(object):
 
         return self
 
-    def __str__(self):
+    def __str__(self) -> str:
         """
         A rendering of the histogram is made using ASCII or unicode characters
         (whatever is supported by the terminal). What exactly is displayed is
@@ -435,26 +502,26 @@ class Histogram(object):
             s = repr(self)
         return s
 
-    def _axis(self, i=0):
+    def _axis(self, i: int = 0) -> Axis:
         """
         Get N-th axis.
         """
         return cast(self, self._hist.axis(i), Axis)
 
     @property
-    def _storage_type(self):
-        return cast(self, self._hist._storage_type, Storage)
+    def _storage_type(self) -> Type[Storage]:
+        return cast(self, self._hist._storage_type, Storage)  # type: ignore
 
-    def _reduce(self, *args):
+    def _reduce(self: H, *args: Any) -> H:
         return self._new_hist(self._hist.reduce(*args))
 
-    def __copy__(self):
+    def __copy__(self: H) -> H:
         return self._new_hist(copy.copy(self._hist))
 
-    def __deepcopy__(self, memo):
+    def __deepcopy__(self: H, memo: Any) -> H:
         return self._new_hist(copy.deepcopy(self._hist), memo=memo)
 
-    def __getstate__(self):
+    def __getstate__(self) -> Tuple[int, Dict[str, Any]]:
         """
         Version 0.8: metadata added
         Version 0.11: version added and set to 0. metadata/_hist replaced with dict.
@@ -467,7 +534,7 @@ class Histogram(object):
         # Version 0 of boost-histogram pickle state
         return (0, local_dict)
 
-    def __setstate__(self, state):
+    def __setstate__(self, state: Any) -> None:
         if isinstance(state, tuple):
             if state[0] == 0:
                 for key, value in state[1].items():
@@ -490,13 +557,13 @@ class Histogram(object):
                 self._hist.axis(i).metadata = {"metadata": self._hist.axis(i).metadata}
             self.axes = self._generate_axes_()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         newline = "\n  "
         sep = "," if len(self.axes) > 0 else ""
         ret = "{self.__class__.__name__}({newline}".format(
             self=self, newline=newline if len(self.axes) > 1 else ""
         )
-        ret += ",{newline}".format(newline=newline).join(repr(ax) for ax in self.axes)
+        ret += f",{newline}".join(repr(ax) for ax in self.axes)
         ret += "{comma}{newline}storage={storage}".format(
             storage=self._storage_type(),
             newline=newline
@@ -510,24 +577,28 @@ class Histogram(object):
         outer = self.sum(flow=True)
         if outer:
             inner = self.sum(flow=False)
-            ret += " # Sum: {}".format(inner)
+            ret += f" # Sum: {inner}"
             if inner != outer:
-                ret += " ({} with flow)".format(outer)
+                ret += f" ({outer} with flow)"
         return ret
 
-    def _compute_commonindex(self, index):
+    def _compute_commonindex(
+        self, index: IndexingExpr
+    ) -> List[Union[SupportsIndex, slice, Mapping[int, Union[SupportsIndex, slice]]]]:
         """
         Takes indices and returns two iterables; one is a tuple or dict of the
         original, Ellipsis expanded index, and the other returns index,
         operation value pairs.
         """
+        indexes: List[Any]
+
         # Shorten the computations with direct access to raw object
         hist = self._hist
 
         # Support dict access
         if hasattr(index, "items"):
             indexes = [slice(None)] * hist.rank()
-            for k, v in index.items():
+            for k, v in index.items():  # type: ignore
                 indexes[k] = v
 
         # Normalize -> h[i] == h[i,]
@@ -549,16 +620,17 @@ class Histogram(object):
             # Note that MyPy doesn't like these very much - the fix
             # will be to properly set input types
             elif callable(indexes[i]):
-                indexes[i] = indexes[i](self.axes[i])  # type: ignore
+                indexes[i] = indexes[i](self.axes[i])
             elif hasattr(indexes[i], "__index__"):
-                if abs(indexes[i]) >= hist.axis(i).size:  # type: ignore
+                if abs(indexes[i]) >= hist.axis(i).size:
                     raise IndexError("histogram index is out of range")
                 indexes[i] %= hist.axis(i).size
 
         return indexes
 
-    @inject_signature("self, flow=False, *, dd=False, view=False")
-    def to_numpy(self, flow=False, **kwargs):
+    def to_numpy(
+        self, flow: bool = False, *, dd: bool = False, view: bool = False
+    ) -> Union[Tuple[np.ndarray, ...], Tuple[np.ndarray, Tuple[np.ndarray, ...]]]:
         """
         Convert to a Numpy style tuple of return arrays. Edges are converted to
         match NumPy standards, with upper edge inclusive, unlike
@@ -585,80 +657,64 @@ class Histogram(object):
             The edges for each dimension
         """
 
-        with KWArgs(kwargs) as kw:
-            dd = kw.optional("dd", False)
-            view = kw.optional("view", False)
-
-        # Python 3+ would be simpler
-        return_tuple = self._hist.to_numpy(flow)
-        hist = return_tuple[0]
-
-        if view:
-            hist = self.view(flow=flow)
-        else:
-            hist = self.values(flow=flow)
+        hist, *edges = self._hist.to_numpy(flow)
+        hist = self.view(flow=flow) if view else self.values(flow=flow)
 
         if dd:
-            return hist, return_tuple[1:]
+            return (hist, edges)
         else:
-            return (hist,) +  return_tuple[1:]
+            return (hist, *edges)
 
-    @inject_signature("self, *, deep=True")
-    def copy(self, **kwargs):
+    def copy(self: H, *, deep: bool = True) -> H:
         """
         Make a copy of the histogram. Defaults to making a
         deep copy (axis metadata copied); use deep=False
         to avoid making a copy of axis metadata.
         """
 
-        # Future versions may add new options here
-        with KWArgs(kwargs) as k:
-            deep = k.optional("deep", True)
-
         if deep:
             return copy.deepcopy(self)
         else:
             return copy.copy(self)
 
-    def reset(self):
+    def reset(self: H) -> H:
         """
         Reset bin counters to default values.
         """
         self._hist.reset()
         return self
 
-    def empty(self, flow=False):
-        # type: (bool) -> bool
+    def empty(self, flow: bool = False) -> bool:
         """
         Check to see if the histogram has any non-default values.
         You can use flow=True to check flow bins too.
         """
-        return self._hist.empty(flow)
+        return self._hist.empty(flow)  # type: ignore
 
-    def sum(self, flow=False):
+    def sum(self, flow: bool = False) -> Union[float, Accumulator]:
         """
         Compute the sum over the histogram bins (optionally including the flow bins).
         """
-        return self._hist.sum(flow)
+        return self._hist.sum(flow)  # type: ignore
 
     @property
-    def size(self):
-        # type: () -> int
+    def size(self) -> int:
         """
         Total number of bins in the histogram (including underflow/overflow).
         """
-        return self._hist.size()
+        return self._hist.size()  # type: ignore
 
     @property
-    def shape(self):
-        # type: () -> Tuple[int, ...]
+    def shape(self) -> Tuple[int, ...]:
         """
         Tuple of axis sizes (not including underflow/overflow).
         """
         return self.axes.size
 
     # TODO: Marked as too complex by flake8. Should be factored out a bit.
-    def __getitem__(self, index):  # noqa: C901
+    def __getitem__(  # noqa: C901
+        self: H, index: IndexingExpr
+    ) -> Union[H, float, Accumulator]:
 
         indexes = self._compute_commonindex(index)
 
@@ -666,7 +722,7 @@ class Histogram(object):
         # But don't try *dict!
         if not hasattr(indexes, "items"):
             try:
-                return self._hist.at(*indexes)
+                return self._hist.at(*indexes)  # type: ignore
             except RuntimeError:
                 pass
 
@@ -676,7 +732,7 @@ class Histogram(object):
         # Compute needed slices and projections
         for i, ind in enumerate(indexes):
             if hasattr(ind, "__index__"):
-                ind = slice(ind.__index__(), ind.__index__() + 1, sum)
+                ind = slice(ind.__index__(), ind.__index__() + 1, sum)  # type: ignore
 
             elif not isinstance(ind, slice):
                 raise IndexError(
@@ -715,6 +771,8 @@ class Histogram(object):
                             "The third argument to a slice must be rebin or projection"
                         )
 
+                assert isinstance(start, int)
+                assert isinstance(stop, int)
                 slices.append(_core.algorithm.slice_and_rebin(i, start, stop, merge))
 
         reduced = self._hist.reduce(*slices)
@@ -730,7 +788,9 @@ class Histogram(object):
                 else reduced.sum(flow=True)
             )
 
-    def __setitem__(self, index, value):
+    def __setitem__(
+        self, index: IndexingExpr, value: Union[ArrayLike, Accumulator]
+    ) -> None:
         """
         There are several supported possibilities:
 
@@ -764,9 +824,9 @@ class Histogram(object):
         # Support raw arrays for accumulators, the final dimension is the constructor values
         if (
             value.ndim > 0
-            and len(view.dtype) > 0
-            and len(value.dtype) == 0
-            and len(view.dtype) == value.shape[-1]
+            and len(view.dtype) > 0  # type: ignore
+            and len(value.dtype) == 0  # type: ignore
+            and len(view.dtype) == value.shape[-1]  # type: ignore
         ):
             value_shape = value.shape[:-1]
             value_ndim = value.ndim - 1
@@ -774,7 +834,7 @@ class Histogram(object):
             value_shape = value.shape
             value_ndim = value.ndim
 
-        # Numpy does not broadcast partial slices, but we would need
+        # NumPy does not broadcast partial slices, but we would need
         # to allow it (because we do allow broadcasting up dimensions)
         # Instead, we simply require matching dimensions.
         if value_ndim > 0 and value_ndim != sum(isinstance(i, slice) for i in indexes):
@@ -821,7 +881,7 @@ class Histogram(object):
                     stop += has_underflow
 
                 else:
-                    msg = "Mismatched shapes in dimension {}".format(n)
+                    msg = f"Mismatched shapes in dimension {n}"
                     msg += ", {} != {}".format(value_shape[n], request_len)
                     if use_underflow or use_overflow:
                         msg += " or {}".format(
@@ -833,10 +893,9 @@ class Histogram(object):
             else:
                 indexes[n] = request + has_underflow
 
-        view[tuple(indexes)] = value
+        view[tuple(indexes)] = value  # type: ignore
 
-    def project(self, *args):
-        # type: (Axis) -> Histogram
+    def project(self: H, *args: int) -> Union[H, float, Accumulator]:
         """
         Project to a single axis or several axes on a multidimensional histogram.
         Provided a list of axis numbers, this will produce the histogram over
@@ -848,8 +907,7 @@ class Histogram(object):
     # Implementation of PlottableHistogram
 
     @property
-    def kind(self):
-        # type: () -> Kind
+    def kind(self) -> Kind:
         """
         Returns Kind.COUNT if this is a normal summing histogram, and Kind.MEAN if this is a
         mean histogram.
@@ -864,8 +922,7 @@ class Histogram(object):
         else:
             return Kind.COUNT
 
-    def values(self, flow=False):
-        # type: (bool) -> ArrayLike
+    def values(self, flow: bool = False) -> np.ndarray:
         """
         Returns the accumulated values. The counts for simple histograms, the
         sum of weights for weighted histograms, the mean for profiles, etc.
@@ -880,13 +937,13 @@ class Histogram(object):
         """
 
         view = self.view(flow)
-        if len(view.dtype) == 0:
+        # TODO: Might be a NumPy typing bug
+        if len(view.dtype) == 0:  # type: ignore
             return view
         else:
-            return view.value
+            return view.value  # type: ignore
 
-    def variances(self, flow=False):
-        # type: (bool) -> Optional[ArrayLike]
+    def variances(self, flow: bool = False) -> Optional[np.ndarray]:
         """
         Returns the estimated variance of the accumulated values. The sum of squared
         weights for weighted histograms, the variance of samples for profiles, etc.
@@ -911,31 +968,30 @@ class Histogram(object):
         """
 
         view = self.view(flow)
-        if len(view.dtype) == 0:
+        if len(view.dtype) == 0:  # type: ignore
             if self._variance_known:
                 return view
             else:
                 return None
         elif hasattr(view, "sum_of_weights"):
-            return np.divide(
-                view.variance,
-                view.sum_of_weights,
-                out=np.full(view.sum_of_weights.shape, np.nan),
-                where=view.sum_of_weights > 1,
+            return np.divide(  # type: ignore
+                view.variance,  # type: ignore
+                view.sum_of_weights,  # type: ignore
+                out=np.full(view.sum_of_weights.shape, np.nan),  # type: ignore
+                where=view.sum_of_weights > 1,  # type: ignore
             )
 
         elif hasattr(view, "count"):
-            return np.divide(
-                view.variance,
-                view.count,
-                out=np.full(view.count.shape, np.nan),
-                where=view.count > 1,
+            return np.divide(  # type: ignore
+                view.variance,  # type: ignore
+                view.count,  # type: ignore
+                out=np.full(view.count.shape, np.nan),  # type: ignore
+                where=view.count > 1,  # type: ignore
             )
         else:
-            return view.variance
+            return view.variance  # type: ignore
 
-    def counts(self, flow=False):
-        # type: (bool) -> Optional[ArrayLike]
+    def counts(self, flow: bool = False) -> np.ndarray:
         """
         Returns the number of entries in each bin for an unweighted
         histogram or profile and an effective number of entries (defined below)
@@ -960,16 +1016,24 @@ class Histogram(object):
 
         view = self.view(flow)
 
-        if len(view.dtype) == 0:
+        if len(view.dtype) == 0:  # type: ignore
             return view
         elif hasattr(view, "sum_of_weights"):
-            return np.divide(
-                view.sum_of_weights ** 2,
-                view.sum_of_weights_squared,
-                out=np.zeros_like(view.sum_of_weights, dtype=np.float64),
-                where=view.sum_of_weights_squared != 0,
+            return np.divide(  # type: ignore
+                view.sum_of_weights ** 2,  # type: ignore
+                view.sum_of_weights_squared,  # type: ignore
+                out=np.zeros_like(view.sum_of_weights, dtype=np.float64),  # type: ignore
+                where=view.sum_of_weights_squared != 0,  # type: ignore
             )
         elif hasattr(view, "count"):
-            return view.count
+            return view.count  # type: ignore
         else:
-            return view.value
+            return view.value  # type: ignore
+
+
+if TYPE_CHECKING:
+    import typing
+
+    from uhi.typing.plottable import PlottableHistogram
+
+    _: PlottableHistogram = typing.cast(Histogram, None)
